@@ -50,43 +50,48 @@ TYPE_SCENARIO = 'SCENARIO'
 TYPE_TIMED_POWER = 'TIMED_POWER'
 TYPE_OUTLET = 'OUTLET'
 
-# List of default skipped types
-SUPPORTED_ITEMS = [TYPE_DIMMER, TYPE_SWITCH, TYPE_SHUTTER]
-
-REQUEST_TIMEOUT = 5
-
-def current_timestamp():
-    return int(datetime.now().timestamp())
-
 
 class SwitchBee():
-    def __init__(self, central_unit, user, password):
+    def __init__(self, central_unit, user, password, websession):
         self.__cunit_ip = central_unit
         self.__user = user
         self.__password = password
         self.__base_url = f'https://{self.__cunit_ip}/{COMMANDS_URL}'
         self.__token = None
-        self.__token_expiration = current_timestamp()
-        self.__tmout = REQUEST_TIMEOUT
-        self.__session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=False), 
-                    timeout=aiohttp.ClientTimeout(total=self.__tmout))
+        self.__token_expiration = 0
+        self.__session = websession
+        self.__login_count = -1
+        self.__relogin_on_invalid_token = True
 
-    async def close(self):
-        await self.__session.close()
+    @property
+    def reconnect_count(self):
+        return self.__login_count
+
+    @property
+    def token_expiration(self):
+        return self.__token_expiration
+
+    @property
+    def relogin_on_invalid_token(self):
+        return self.__relogin_on_invalid_token
+    
+    @relogin_on_invalid_token.setter
+    def relogin_on_invalid_token(self, value: bool):
+        self.__relogin_on_invalid_token = value
 
     async def __send_request(self, command, params={}):            
         async def do_send_request(payload):
             try:
                 resp = await self.__session.post(url=self.__base_url, json=payload)
             except asyncio.TimeoutError as e:
-                return {ATTR_STATUS: STATUS_FAILED, "code": -200, 'message': e}
+                return {ATTR_STATUS: STATUS_FAILED, 'message': e}
             except aiohttp.ClientError as e:
-                return {ATTR_STATUS: STATUS_FAILED, "code": -200, 'message': e}
-            
+                return {ATTR_STATUS: STATUS_FAILED, 'message': e}
             try:
                 return await resp.json(content_type=None)
             except JSONDecodeError as e:
-                return {ATTR_STATUS: STATUS_FAILED, "code": -200, 'message': 'Unexpected Response'}
+                resp = await resp.read()
+                return {ATTR_STATUS: STATUS_FAILED, 'message': 'Unexpected Response', 'response': resp}
 
         if command == CMD_LOGIN:
             payload = {
@@ -105,8 +110,8 @@ class SwitchBee():
             }
 
         resp = await do_send_request(payload)
-        # Someone else must've logged in and refreshed the token, for now just try to log in again
-        if resp[ATTR_STATUS] != STATUS_OK and resp[ATTR_STATUS] == STATUS_INVALID_TOKEN:
+        # Someone else logged in with the same user and refreshed the token, for now just try to login again
+        if self.__relogin_on_invalid_token and resp[ATTR_STATUS] != STATUS_OK and resp[ATTR_STATUS] == STATUS_INVALID_TOKEN:
             self.login()
             resp = await do_send_request(payload)
         
@@ -115,6 +120,7 @@ class SwitchBee():
     async def login(self):
         resp = await self.__send_request(CMD_LOGIN)
         if resp[ATTR_STATUS] == STATUS_OK:
+            self.__login_count += 1
             self.__token = resp[ATTR_DATA][ATTR_TOKEN]
             self.__token_expiration = resp[ATTR_DATA][ATTR_EXPIRATION]
         
